"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Babel = require("@babel/core");
const fs_1 = require("fs");
const glob_1 = require("glob");
const path_1 = require("path");
const resolve_1 = require("resolve");
const source_map_support_1 = require("source-map-support");
const PluginLoader_1 = require("./PluginLoader");
const RecastPlugin_1 = require("./RecastPlugin");
const AstExplorerResolver_1 = require("./resolvers/AstExplorerResolver");
const FileSystemResolver_1 = require("./resolvers/FileSystemResolver");
const NetworkResolver_1 = require("./resolvers/NetworkResolver");
const PackageResolver_1 = require("./resolvers/PackageResolver");
const transpile_requires_1 = require("./transpile-requires");
exports.DEFAULT_EXTENSIONS = new Set(['.js', '.jsx']);
class Plugin {
    constructor(rawPlugin, inferredName, source, resolvedPath) {
        this.rawPlugin = rawPlugin;
        this.inferredName = inferredName;
        this.source = source;
        this.resolvedPath = resolvedPath;
        let instance = rawPlugin(Babel);
        if (instance.name) {
            this.declaredName = instance.name;
        }
    }
}
exports.Plugin = Plugin;
class Options {
    constructor(sourcePaths, localPlugins, remotePlugins, pluginOptions, extensions, requires, transpilePlugins, findBabelConfig, ignore, stdio, help, dry) {
        this.sourcePaths = sourcePaths;
        this.localPlugins = localPlugins;
        this.remotePlugins = remotePlugins;
        this.pluginOptions = pluginOptions;
        this.extensions = extensions;
        this.requires = requires;
        this.transpilePlugins = transpilePlugins;
        this.findBabelConfig = findBabelConfig;
        this.ignore = ignore;
        this.stdio = stdio;
        this.help = help;
        this.dry = dry;
        this.pluginLoader = new PluginLoader_1.default([
            new FileSystemResolver_1.default(),
            new PackageResolver_1.default()
        ]);
        this.remotePluginLoader = new PluginLoader_1.default([
            new AstExplorerResolver_1.default(),
            new NetworkResolver_1.default()
        ]);
    }
    getPlugins() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._pluginCache) {
                let localPlugins = Promise.all(this.localPlugins.map((localPlugin) => __awaiter(this, void 0, void 0, function* () {
                    let pluginExports = yield this.pluginLoader.load(localPlugin);
                    let defaultExport = pluginExports['default'] || pluginExports;
                    return new Plugin(defaultExport, path_1.basename(localPlugin, path_1.extname(localPlugin)));
                })));
                let remotePlugins = Promise.all(this.remotePlugins.map((remotePlugin) => __awaiter(this, void 0, void 0, function* () {
                    let pluginExports = yield this.remotePluginLoader.load(remotePlugin);
                    let defaultExport = pluginExports['default'] || pluginExports;
                    return new Plugin(defaultExport, path_1.basename(remotePlugin, path_1.extname(remotePlugin)));
                })));
                this._pluginCache = [...(yield localPlugins), ...(yield remotePlugins)];
            }
            return this._pluginCache;
        });
    }
    loadRequires() {
        for (let modulePath of this.requires) {
            require(modulePath);
        }
    }
    loadBabelTranspile() {
        if (this.transpilePlugins) {
            transpile_requires_1.enable(this.findBabelConfig);
            source_map_support_1.install();
        }
    }
    unloadBabelTranspile() {
        if (this.transpilePlugins) {
            transpile_requires_1.disable();
        }
    }
    getPlugin(name) {
        return __awaiter(this, void 0, void 0, function* () {
            for (let plugin of yield this.getPlugins()) {
                if (plugin.declaredName === name || plugin.inferredName === name) {
                    return plugin;
                }
            }
            return null;
        });
    }
    getBabelPlugins() {
        return __awaiter(this, void 0, void 0, function* () {
            let result = [RecastPlugin_1.default];
            for (let plugin of yield this.getPlugins()) {
                let options = (plugin.declaredName && this.pluginOptions.get(plugin.declaredName)) ||
                    this.pluginOptions.get(plugin.inferredName);
                if (options) {
                    result.push([plugin.rawPlugin, options]);
                }
                else {
                    result.push(plugin.rawPlugin);
                }
            }
            return result;
        });
    }
    getBabelPlugin(name) {
        return __awaiter(this, void 0, void 0, function* () {
            let plugin = yield this.getPlugin(name);
            if (!plugin) {
                return null;
            }
            let options = this.pluginOptions.get(name);
            if (options) {
                return [plugin.rawPlugin, options];
            }
            else {
                return plugin.rawPlugin;
            }
        });
    }
    static parse(args) {
        let sourcePaths = [];
        let localPlugins = [];
        let remotePlugins = [];
        let pluginOptions = new Map();
        let extensions = exports.DEFAULT_EXTENSIONS;
        let ignore = (path, basename, root) => basename[0] === '.';
        let requires = [];
        let findBabelConfig = false;
        let transpilePlugins = true;
        let stdio = false;
        let help = false;
        let dry = false;
        for (let i = 0; i < args.length; i++) {
            let arg = args[i];
            switch (arg) {
                case '-p':
                case '--plugin':
                    i++;
                    localPlugins.push(args[i]);
                    break;
                case '--remote-plugin':
                    i++;
                    remotePlugins.push(args[i]);
                    break;
                case '-o':
                case '--plugin-options':
                    i++;
                    let nameAndOptions = args[i].split('=');
                    let name = nameAndOptions[0];
                    let optionsRaw = nameAndOptions[1];
                    if (optionsRaw && optionsRaw[0] === '@') {
                        optionsRaw = fs_1.readFileSync(optionsRaw.slice(1), {
                            encoding: 'utf8'
                        });
                        pluginOptions.set(name, JSON.parse(optionsRaw));
                    }
                    try {
                        pluginOptions.set(name, JSON.parse(optionsRaw));
                    }
                    catch (err) {
                        return new Error(`unable to parse JSON config for ${name}: ${optionsRaw}`);
                    }
                    break;
                case '-r':
                case '--require':
                    i++;
                    requires.push(getRequirableModulePath(args[i]));
                    break;
                case '--transpile-plugins':
                case '--no-transpile-plugins':
                    transpilePlugins = arg === '--transpile-plugins';
                    break;
                case '--find-babel-config':
                case '--no-find-babel-config':
                    findBabelConfig = arg === '--find-babel-config';
                    break;
                case '--extensions':
                    i++;
                    extensions = new Set(args[i].split(',').map(ext => (ext[0] === '.' ? ext : `.${ext}`)));
                    break;
                case '-s':
                case '--stdio':
                    stdio = true;
                    break;
                case '-h':
                case '--help':
                    help = true;
                    break;
                case '-d':
                case '--dry':
                    dry = true;
                    break;
                default:
                    if (arg[0] === '-') {
                        return new Error(`unexpected option: ${arg}`);
                    }
                    else {
                        if (glob_1.hasMagic(arg)) {
                            sourcePaths.push(...glob_1.sync(arg));
                        }
                        else {
                            sourcePaths.push(arg);
                        }
                    }
                    break;
            }
        }
        return new Options(sourcePaths, localPlugins, remotePlugins, pluginOptions, extensions, requires, transpilePlugins, findBabelConfig, ignore, stdio, help, dry);
    }
}
exports.default = Options;
/**
 * Gets a path that can be passed to `require` for a given module path.
 */
function getRequirableModulePath(modulePath) {
    if (fs_1.existsSync(modulePath) || fs_1.existsSync(modulePath + '.js')) {
        return path_1.resolve(modulePath);
    }
    else {
        return resolve_1.sync(modulePath, { basedir: process.cwd() });
    }
}
//# sourceMappingURL=Options.js.map