"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const realFs = require("fs");
const getStream = require("get-stream");
const path_1 = require("path");
const iterateSources_1 = require("./iterateSources");
const Options_1 = require("./Options");
const ProcessSnapshot_1 = require("./ProcessSnapshot");
const TransformRunner_1 = require("./TransformRunner");
function printHelp(argv, out) {
    let $0 = path_1.basename(argv[1]);
    out.write(`
${$0} [OPTIONS] [PATH … | --stdio]

OPTIONS
  -p, --plugin PLUGIN               Transform sources with PLUGIN (allows multiple).
      --remote-plugin URL           Fetch a plugin from URL (allows multiple).
  -o, --plugin-options PLUGIN=OPTS  JSON-encoded OPTS for PLUGIN (allows multiple).
  -r, --require PATH                Require PATH before transform (allows multiple).
      --extensions EXTS             Comma-separated extensions to process (default: "${Array.from(Options_1.DEFAULT_EXTENSIONS).join(',')}").
      --[no-]transpile-plugins      Transpile plugins to enable future syntax (default: on).
      --[no-]find-babel-config      Run plugins through babel plugins/presets specified in local
                                    .babelrc file instead of babel-preset-env (default: off).
  -s, --stdio                       Read source from stdin and print to stdout.
  -h, --help                        Show this help message.
  -d, --dry                         Run plugins without modifying files on disk.

  NOTE: \`--remote-plugin\` should only be used as a convenience to load code that you or someone
        you trust wrote. It will run with your full user privileges, so please exercise caution!

EXAMPLES
  # Run with a relative plugin on all files in \`src/\`.
  $ ${$0} -p ./typecheck.js src/

  # Run with a remote plugin from astexplorer.net on all files in \`src/\`.
  $ ${$0} --remote-plugin 'https://astexplorer.net/#/gist/688274…' src/

  # Run with multiple plugins.
  $ ${$0} -p ./a.js -p ./b.js some-file.js

  # Run with a plugin in \`node_modules\` on stdin.
  $ ${$0} -s -p babel-plugin-typecheck <<EOS
  function add(a: number, b: number): number {
    return a + b;
  }
  EOS

  # Pass options to a plugin.
  $ ${$0} -p ./a.js -o a='{"foo":true}' src/

  # Pass options from a config file to a plugin.
  $ ${$0} -p ./a.js -o a=@opts.json src/

  # Run with a plugin written in TypeScript.
  $ ${$0} -p ./some-plugin.ts src/
  `.trim());
    out.write('\n');
}
function run(argv, stdin, stdout, stderr, fs = realFs) {
    return __awaiter(this, void 0, void 0, function* () {
        let options = Options_1.default.parse(argv.slice(2));
        if (options instanceof Error) {
            stderr.write(`ERROR: ${options.message}\n`);
            printHelp(argv, stderr);
            return 1;
        }
        if (options.help) {
            printHelp(argv, stdout);
            return 0;
        }
        let snapshot = new ProcessSnapshot_1.default();
        let plugins;
        try {
            options.loadBabelTranspile();
            options.loadRequires();
            plugins = yield options.getBabelPlugins();
        }
        finally {
            options.unloadBabelTranspile();
            snapshot.restore();
        }
        let runner;
        let stats = {
            modified: 0,
            total: 0,
            errors: 0
        };
        let dryRun = options.dry;
        let sourcesIterator;
        if (options.stdio) {
            sourcesIterator = [new TransformRunner_1.Source('<stdin>', yield getStream(stdin))][Symbol.iterator]();
        }
        else {
            sourcesIterator = iterateSources_1.default(options.sourcePaths, options.extensions, options.ignore, fs.statSync, fs.readdirSync, fs.readFileSync);
        }
        runner = new TransformRunner_1.default(sourcesIterator, plugins);
        let dim = stdout.isTTY ? '\x1b[2m' : '';
        let reset = stdout.isTTY ? '\x1b[0m' : '';
        for (let result of runner.run()) {
            if (result.output) {
                if (options.stdio) {
                    stdout.write(`${result.output}\n`);
                }
                else {
                    if (result.output === result.source.content) {
                        stdout.write(`${dim}${result.source.path}${reset}\n`);
                    }
                    else {
                        stats.modified++;
                        stdout.write(`${result.source.path}\n`);
                        if (!dryRun) {
                            fs.writeFileSync(result.source.path, result.output);
                        }
                    }
                }
            }
            else if (result.error) {
                stats.errors++;
                if (!options.stdio) {
                    stderr.write(`Encountered an error while processing ${result.source.path}:\n`);
                }
                stderr.write(`${result.error.stack}\n`);
            }
            stats.total++;
        }
        if (!options.stdio) {
            if (dryRun) {
                stdout.write('DRY RUN: no files affected\n');
            }
            stdout.write(`${stats.total} file(s), ${stats.modified} modified, ${stats.errors} errors\n`);
        }
        // exit status is number of errors up to byte max value
        return Math.min(stats.errors, 255);
    });
}
exports.default = run;
//# sourceMappingURL=index.js.map